// Production 17 Function DCC Decoder   Dec_17LED_1Ftn.ino
// Version 6.01  Geoff Bunza 2014,2015,2016,2017,2018
// Now works with both short and long DCC Addesses
//
// ******** UNLESS YOU WANT ALL CV'S RESET UPON EVERY POWER UP
// ******** AFTER THE INITIAL DECODER LOAD REMOVE THE "//" IN THE FOOLOWING LINE!!
// #define DECODER_LOADED
//
//  ---
//  Anpassung an Arduino Nano                                               JoFri 2023-10-24
//  initialisieren der Relais ausgeschaltet
//  Anpassung an WildeMaus
//  Sketch Name: Wmaus_Dec_17LED_1Ftn.ino
//  Umbau von DCC_Decoder.h nach NmraDcc.h  auf WildeMaus 31.10.2023 -       Wipf 2023-10-31
//
//  benötigt wird die NmraDcc-master.zip   NmraDcc.h
//  Diese Bibliothek ist auf neuerem Stand unter:  https://github.com/mrrwa/NmraDcc
//
#include <NmraDcc.h>

#define This_Decoder_Address 7  // DCC Decoder Adresse CV 1
#define numleds 17              // Anzahl der Ausgänge 17

// Pin Definition
#define PIN_SYSTEM 13    // 13 | F0 LED auf Arduino NANO
#define PIN_MOTOR 14     // A0 | F1 Motor
#define PIN_LOGO 15      // A1 | F2 WildeMaus Dauerlicht, F5 Blinken WildeMaus
#define PIN_HAUS 16      // A2 | F3 Licht 1 Häuschen
#define PIN_LATERNEN 17  // A3 | F4 Licht 2 Laternen
#define PIN_BAND_A 3     // D3 | F6, F7, F8, F9, F10 Lauflicht verschiedene Modi
#define PIN_BAND_B 4     // D4 | F6, F7, F8, F9, F10 Lauflicht verschiedene Modi
#define PIN_BAND_C 5     // D5 | F6, F7, F8, F9, F10 Lauflicht verschiedene Modi

#define ANIMATION_MAX 12  // Animationszähler Lauflichtlicht Takt

NmraDcc Dcc;
DCC_MSG Packet;
uint8_t CV_DECODER_MASTER_RESET = 120;
int nTimerBlink = 0;
int nTimerNextStep = 0;
int nAnimatonSelect = 0;
boolean bF0, bF1, bF2, bF3, bF4, bF5, bF6, bF7, bF8, bF9, bF10, bF11 = false;

struct CVPair {
  uint16_t CV;
  uint8_t Value;
};

CVPair FactoryDefaultCVs[] =

  {
    { CV_MULTIFUNCTION_PRIMARY_ADDRESS, This_Decoder_Address & 0x7F },

    // These two CVs define the Long DCC Address
    { CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB, ((This_Decoder_Address >> 8) & 0x7F) + 192 },
    { CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB, This_Decoder_Address & 0xFF },

    // ONLY uncomment 1 CV_29_CONFIG line below as approprate DEFAULT IS SHORT ADDRESS
    //  {CV_29_CONFIG,          0},                               // Short Address 14 Speed Steps
    { CV_29_CONFIG, CV29_F0_LOCATION },                           // Short Address 28/128 Speed Steps
    //  {CV_29_CONFIG, CV29_EXT_ADDRESSING | CV29_F0_LOCATION},   // Long  Address 28/128 Speed Steps

    { CV_DECODER_MASTER_RESET, 0 },
  };

uint8_t FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs) / sizeof(CVPair);

void notifyCVResetFactoryDefault() {
  // Make FactoryDefaultCVIndex non-zero and equal to num CV's to be reset
  // to flag to the loop() function that a reset to Factory Defaults needs to be done
  FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs) / sizeof(CVPair);
};

void setup() {
  // initialize the digital pins as an outputs
  for (int i = 2; i <= numleds; i++) {
    pinMode(i + 1, OUTPUT);
    digitalWrite(i + 1, HIGH);  // war LOW 
  }

  // Setup which External Interrupt, the Pin it's associated with that we're using and enable the Pull-Up
  Dcc.pin(0, 2, 0);
  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init(MAN_ID_DIY, 601, FLAGS_MY_ADDRESS_ONLY, 0);
  delay(800);
#if defined(DECODER_LOADED)
  if (Dcc.getCV(CV_DECODER_MASTER_RESET) == CV_DECODER_MASTER_RESET)
#endif
  {
    for (int j = 0; j < FactoryDefaultCVIndex; j++)
      Dcc.setCV(FactoryDefaultCVs[j].CV, FactoryDefaultCVs[j].Value);
    digitalWrite(13, 1);  //Blink the on board LED
    delay(1000);
    digitalWrite(13, 0);
  }
}

void loop() {
  // You MUST call the NmraDcc.process() method frequently from the Arduino loop() function for correct library operation
  Dcc.process();
  if (FactoryDefaultCVIndex && Dcc.isSetCVReady()) {
    FactoryDefaultCVIndex--;  // Decrement first as initially it is the size of the array
    Dcc.setCV(FactoryDefaultCVs[FactoryDefaultCVIndex].CV, FactoryDefaultCVs[FactoryDefaultCVIndex].Value);
  }

  if (nTimerNextStep < 1000) {  // von 0 bis 999 -  Animationszähler Lauflichtlicht Takt Verzögerung
    nTimerNextStep++;
  } else {
    nTimerNextStep = 0;
    if (nTimerBlink < ANIMATION_MAX) {  // von 0 bis 11 Steps -  Animationszähler Lauflichtlicht Takt
      nTimerBlink++;
    } else {
      nTimerBlink = 0;
    }
  }

  doSystemLED();
  doMotor();
  doWildeMausLogo();
  doHausLichter();
  doLichtband();
}

void notifyDccFunc(uint16_t Addr, DCC_ADDR_TYPE AddrType, FN_GROUP FuncGrp, uint8_t FuncState) {
  switch (FuncGrp) {
    case FN_0_4:
      bF0 = ((FuncState & FN_BIT_00) >> 4);  // F0  LED auf Arduino NANO

      bF1 = ((FuncState & FN_BIT_01));       // F1  Motor ein/aus                            
      bF2 = ((FuncState & FN_BIT_02) >> 1);  // F2  WildeMaus Anzeige
      bF3 = ((FuncState & FN_BIT_03) >> 2);  // F3  Licht 1 Häuschen
      bF4 = ((FuncState & FN_BIT_04) >> 3);  // F4  Licht 2 Laternen
      break;

    case FN_5_8:
      bF5 = ((FuncState & FN_BIT_05));       // F5  blinken von Wilde Maus
      bF6 = ((FuncState & FN_BIT_06) >> 1);  // F6  Lauflicht alle Animationen
      bF7 = ((FuncState & FN_BIT_07) >> 2);  // F7  Lauflicht Animation2 einfach nach links
      bF8 = ((FuncState & FN_BIT_08) >> 3);  // F8  Lauflicht Animation3 einfach nach rechts
      break;

    case FN_9_12:
      bF9 = ((FuncState & FN_BIT_09));        // F9  Lauflicht Animation4 doppelt nach links
      bF10 = ((FuncState & FN_BIT_10) >> 1);  // F10 Lauflicht alle Leuchtpunkte ein/aus
      bF11 = ((FuncState & FN_BIT_11) >> 2);  // F11 Lauflicht Blitzlicht
      //   bF12 = ((FuncState & FN_BIT_12) >> 3);  // F12 nicht verwendet
      //   break;

      // case FN_13_20:
      //   bF13 = ((FuncState & FN_BIT_13));       // F13  nicht verwendet       HIGH = aktiv
      //   bF14 = ((FuncState & FN_BIT_14) >> 1);  // F14  nicht verwendet
      //   bF15 = ((FuncState & FN_BIT_15) >> 2);  // F15  nicht verwendet
      //   bF16 = ((FuncState & FN_BIT_16) >> 3);  // F16  nicht verwendet
      //   break;

      // case FN_21_28:
      break;
  }
}

void animation1() {  // F6 alle Lauflicht Animationen hintereinander
  if (nTimerBlink == ANIMATION_MAX) {
    if (nAnimatonSelect > 14) {
      nAnimatonSelect = 0;
    } else {
      nAnimatonSelect++;
    }
  }

  if (nAnimatonSelect == 0 || nAnimatonSelect == 1) {
    animation2();
  } else if (nAnimatonSelect == 2 || nAnimatonSelect == 3) {
    animation3();
  } else if (nAnimatonSelect == 5 || nAnimatonSelect == 6) {
    animation5();
  } else if (nAnimatonSelect >= 7) {
    animation4();
  }
}

void animation2() {  // F7 Lauflicht einfach nach links
  if (nTimerBlink == 0)
    lichtband100();
  if (nTimerBlink == 4)
    lichtband010();
  if (nTimerBlink == 8)
    lichtband001();
}

void animation3() {  // F8 Lauflicht einfach nach rechts
  if (nTimerBlink == 0)
    lichtband001();
  if (nTimerBlink == 4)
    lichtband010();
  if (nTimerBlink == 8)
    lichtband100();
}

void animation4() {  // F9 Lauflicht doppelt nach links
  if (nTimerBlink == 0)
    lichtband110();
  if (nTimerBlink == 4)
    lichtband011();
  if (nTimerBlink == 8)
    lichtband101();
}

void animation5() {  // F11 Lauflicht - Blitzlicht
  if (nTimerBlink % 2 == 0) {
    lichtband111();
  } else {
    lichtband000();
  }
}

void lichtband000() {  // HIGH aus, LOW aktiv, alle aus
  digitalWrite(PIN_BAND_A, HIGH);
  digitalWrite(PIN_BAND_B, HIGH);
  digitalWrite(PIN_BAND_C, HIGH);
}
void lichtband001() {
  digitalWrite(PIN_BAND_A, HIGH);
  digitalWrite(PIN_BAND_B, HIGH);
  digitalWrite(PIN_BAND_C, LOW);
}
void lichtband010() {
  digitalWrite(PIN_BAND_A, HIGH);
  digitalWrite(PIN_BAND_B, LOW);
  digitalWrite(PIN_BAND_C, HIGH);
}
void lichtband011() {
  digitalWrite(PIN_BAND_A, HIGH);
  digitalWrite(PIN_BAND_B, LOW);
  digitalWrite(PIN_BAND_C, LOW);
}
void lichtband100() {
  digitalWrite(PIN_BAND_A, LOW);
  digitalWrite(PIN_BAND_B, HIGH);
  digitalWrite(PIN_BAND_C, HIGH);
}
void lichtband101() {
  digitalWrite(PIN_BAND_A, LOW);
  digitalWrite(PIN_BAND_B, HIGH);
  digitalWrite(PIN_BAND_C, LOW);
}
void lichtband110() {
  digitalWrite(PIN_BAND_A, LOW);
  digitalWrite(PIN_BAND_B, LOW);
  digitalWrite(PIN_BAND_C, HIGH);
}
void lichtband111() {
  digitalWrite(PIN_BAND_A, LOW);
  digitalWrite(PIN_BAND_B, LOW);
  digitalWrite(PIN_BAND_C, LOW);
}

void doLichtband() {
  if (bF10) {  // Wenn F10 an ist, keine Animation, alle Leuchtpunkte an
    lichtband111();
  } else {
    if (bF6) {
      animation1();
    } else if (bF7) {
      animation2();
    } else if (bF8) {
      animation3();
    } else if (bF9) {
      animation4();
    } else if (bF11) {
      animation5();
    } else {  // Weder F10 noch Animation -> Lauflicht aus
      lichtband000();
    }
  }
}

void doSystemLED() {  // LED auf Nano
  digitalWrite(PIN_SYSTEM, bF0);
}

void doMotor() {  // Motor
  digitalWrite(PIN_MOTOR, bF1);
}

void doHausLichter() {  // HausLichter, Laternen
  digitalWrite(PIN_HAUS, bF3);
  digitalWrite(PIN_LATERNEN, bF4);
}

void doWildeMausLogo() {  // WildeMausLogo, F2 immer an
  if (bF2) {
    digitalWrite(PIN_LOGO, HIGH);
  } else {  // Wenn kein F2, prüfe auf F5
    if (bF5 && (nTimerBlink > 3)) {
      digitalWrite(PIN_LOGO, HIGH);
    } else {
      digitalWrite(PIN_LOGO, LOW);
    }
  }
}
